@@ -9,14 +9,15 @@ from __future__ import annotations
 import re
 import struct
 from math import prod
-from typing import Any, Callable, Pattern, cast
+from typing import Any, cast
+from functools import cache
+from collections.abc import Callable
 
 from .. import errors as e
 from .. import postgres, pq
 from ..abc import AdaptContext, Buffer, Dumper, DumperKey, Loader, NoneType, Transformer
 from .._oids import INVALID_OID, TEXT_ARRAY_OID, TEXT_OID
 from ..adapt import PyFormat, RecursiveDumper, RecursiveLoader
-from .._compat import cache
 from .._struct import pack_len, unpack_len
 from .._cmodule import _psycopg
 from .._typeinfo import TypeInfo
@@ -33,6 +34,8 @@ _unpack_dim = cast(Callable[[Buffer, int], "tuple[int, int]"], _struct_dim.unpac
 PY_TEXT = PyFormat.TEXT
 PQ_BINARY = pq.Format.BINARY
 
+ARRAY_NULL = b"\xff\xff\xff\xff"
+
 
 class BaseListDumper(RecursiveDumper):
     element_oid = INVALID_OID
@@ -52,8 +55,7 @@ class BaseListDumper(RecursiveDumper):
         Find the first non-null element of an eventually nested list
         """
         items = list(self._flatiter(L, set()))
-        types = {type(item): item for item in items}
-        if not types:
+        if not (types := {type(item): item for item in items}):
             return None
 
         if len(types) == 1:
@@ -62,8 +64,7 @@ class BaseListDumper(RecursiveDumper):
             # More than one type in the list. It might be still good, as long
             # as they dump with the same oid (e.g. IPv4Network, IPv6Network).
             dumpers = [self._tx.get_dumper(item, format) for item in types.values()]
-            oids = {d.oid for d in dumpers}
-            if len(oids) == 1:
+            if len({d.oid for d in dumpers}) == 1:
                 t, v = types.popitem()
             else:
                 raise e.DataError(
@@ -106,8 +107,7 @@ class BaseListDumper(RecursiveDumper):
         Return text info as fallback.
         """
         if base_oid:
-            info = self._tx.adapters.types.get(base_oid)
-            if info:
+            if info := self._tx.adapters.types.get(base_oid):
                 return info
 
         return self._tx.adapters.types["text"]
@@ -120,8 +120,7 @@ class ListDumper(BaseListDumper):
         if self.oid:
             return self.cls
 
-        item = self._find_list_element(obj, format)
-        if item is None:
+        if (item := self._find_list_element(obj, format)) is None:
             return self.cls
 
         sd = self._tx.get_dumper(item, format)
@@ -132,8 +131,7 @@ class ListDumper(BaseListDumper):
         if self.oid:
             return self
 
-        item = self._find_list_element(obj, format)
-        if item is None:
+        if (item := self._find_list_element(obj, format)) is None:
             # Empty lists can only be dumped as text if the type is unknown.
             return self
 
@@ -170,8 +168,7 @@ class ListDumper(BaseListDumper):
                 if isinstance(item, list):
                     dump_list(item)
                 elif item is not None:
-                    ad = self._dump_item(item)
-                    if ad is None:
+                    if (ad := self._dump_item(item)) is None:
                         tokens.append(b"NULL")
                     else:
                         if needs_quotes(ad):
@@ -198,7 +195,7 @@ class ListDumper(BaseListDumper):
 
 
 @cache
-def _get_needs_quotes_regexp(delimiter: bytes) -> Pattern[bytes]:
+def _get_needs_quotes_regexp(delimiter: bytes) -> re.Pattern[bytes]:
     """Return a regexp to recognise when a value needs quotes
 
     from https://www.postgresql.org/docs/current/arrays.html#ARRAYS-IO
@@ -224,8 +221,7 @@ class ListBinaryDumper(BaseListDumper):
         if self.oid:
             return self.cls
 
-        item = self._find_list_element(obj, format)
-        if item is None:
+        if (item := self._find_list_element(obj, format)) is None:
             return (self.cls,)
 
         sd = self._tx.get_dumper(item, format)
@@ -236,8 +232,7 @@ class ListBinaryDumper(BaseListDumper):
         if self.oid:
             return self
 
-        item = self._find_list_element(obj, format)
-        if item is None:
+        if (item := self._find_list_element(obj, format)) is None:
             return ListDumper(self.cls, self._tx)
 
         sd = self._tx.get_dumper(item, format.from_pq(self.format))
@@ -283,7 +278,7 @@ class ListBinaryDumper(BaseListDumper):
                         data.append(item)
                     else:
                         hasnull = 1
-                        data.append(b"\xff\xff\xff\xff")
+                        data.append(ARRAY_NULL)
             else:
                 for item in L:
                     if not isinstance(item, self.cls):
@@ -384,7 +379,7 @@ def _load_text(
     data: Buffer,
     loader: Loader,
     delimiter: bytes = b",",
-    __re_unescape: Pattern[bytes] = re.compile(rb"\\(.)"),
+    __re_unescape: re.Pattern[bytes] = re.compile(rb"\\(.)"),
 ) -> list[Any]:
     rv = None
     stack: list[Any] = []
@@ -396,15 +391,14 @@ def _load_text(
     if data and data[0] == b"["[0]:
         if isinstance(data, memoryview):
             data = bytes(data)
-        idx = data.find(b"=")
-        if idx == -1:
+
+        if (idx := data.find(b"=")) == -1:
             raise e.DataError("malformed array: no '=' after dimension information")
         data = data[idx + 1 :]
 
     re_parse = _get_array_parse_regexp(delimiter)
     for m in re_parse.finditer(data):
-        t = m.group(1)
-        if t == b"{":
+        if (t := m.group(1)) == b"{":
             if stack:
                 stack[-1].append(a)
             stack.append(a)
@@ -433,7 +427,7 @@ def _load_text(
 
 
 @cache
-def _get_array_parse_regexp(delimiter: bytes) -> Pattern[bytes]:
+def _get_array_parse_regexp(delimiter: bytes) -> re.Pattern[bytes]:
     """
     Return a regexp to tokenize an array representation into item and brackets
     """
