@@ -9,13 +9,13 @@ from random import choice, random, randrange
 from typing import Any
 from decimal import Decimal
 from contextlib import asynccontextmanager, contextmanager
+from collections import deque
 
 import pytest
 
 import psycopg
 from psycopg import sql
 from psycopg.adapt import PyFormat
-from psycopg._compat import Deque
 from psycopg.types.json import Json, Jsonb
 from psycopg.types.range import Range
 from psycopg.types.numeric import Int4, Int8
@@ -157,8 +157,7 @@ class Faker:
                     try:
                         cur.execute(self._insert_field_stmt(j), (val,))
                     except psycopg.DatabaseError as e:
-                        r = repr(val)
-                        if len(r) > 200:
+                        if len(r := repr(val)) > 200:
                             r = f"{r[:200]}... ({len(r)} chars)"
                         raise Exception(
                             f"value {r!r} at record {i} column0 {j} failed insert: {e}"
@@ -182,8 +181,7 @@ class Faker:
                     try:
                         await acur.execute(self._insert_field_stmt(j), (val,))
                     except psycopg.DatabaseError as e:
-                        r = repr(val)
-                        if len(r) > 200:
+                        if len(r := repr(val)) > 200:
                             r = f"{r[:200]}... ({len(r)} chars)"
                         raise Exception(
                             f"value {r!r} at record {i} column0 {j} failed insert: {e}"
@@ -201,8 +199,7 @@ class Faker:
     def choose_schema(self, ncols=20):
         schema: list[tuple[type, ...] | type] = []
         while len(schema) < ncols:
-            s = self.make_schema(choice(self.types))
-            if s is not None:
+            if (s := self.make_schema(choice(self.types))) is not None:
                 schema.append(s)
         self.schema = schema
         return schema
@@ -273,8 +270,7 @@ class Faker:
         except KeyError:
             pass
 
-        meth = self._get_method("make", cls)
-        if meth:
+        if meth := self._get_method("make", cls):
             self._makers[cls] = meth
             return meth
         else:
@@ -292,9 +288,7 @@ class Faker:
 
         parts = name.split(".")
         for i in range(len(parts) - 1, -1, -1):
-            mname = f"{prefix}_{'_'.join(parts[-(i + 1) :])}"
-            meth = getattr(self, mname, None)
-            if meth:
+            if meth := getattr(self, f"{prefix}_{'_'.join(parts[-(i + 1):])}", None):
                 return meth
 
         return None
@@ -306,8 +300,7 @@ class Faker:
     def example(self, spec):
         # A good representative of the object - no degenerate case
         cls = spec if isinstance(spec, type) else spec[0]
-        meth = self._get_method("example", cls)
-        if meth:
+        if meth := self._get_method("example", cls):
             return meth(spec)
         else:
             return self.make(spec)
@@ -406,11 +399,11 @@ class Faker:
         if random() <= 0.99:
             # These exponents should generate no inf/overflow
             if size == 64:
-                s = f"{choice('-+')}0.{randrange(1 << 53)}e{randrange(-310,309)}"
+                s = f"{choice('-+')}0.{randrange(1 << 53)}e{randrange(-310, 309)}"
             elif size == 32:
-                s = f"{choice('-+')}0.{randrange(1 << 22)}e{randrange(-37,38)}"
+                s = f"{choice('-+')}0.{randrange(1 << 22)}e{randrange(-37, 38)}"
             elif size == 16:
-                s = f"{choice('-+')}0.{randrange(1 << 10)}e{randrange(-3,4)}"
+                s = f"{choice('-+')}0.{randrange(1 << 10)}e{randrange(-3, 4)}"
             else:
                 assert False, size
             return spec(s)
@@ -422,11 +415,10 @@ class Faker:
     def match_float(self, spec, got, want, rel=None):
         if got is not None and isnan(got):
             assert isnan(want)
+        elif rel or self._server_rounds():
+            assert got == pytest.approx(want, rel=rel)
         else:
-            if rel or self._server_rounds():
-                assert got == pytest.approx(want, rel=rel)
-            else:
-                assert got == want
+            assert got == want
 
     def _server_rounds(self):
         """Return True if the connected server perform float rounding"""
@@ -505,25 +497,20 @@ class Faker:
     def make_JsonFloat(self, spec):
         # A float limited to what json accepts
         # this exponent should generate no inf
-        return float(f"{choice('-+')}0.{randrange(1 << 20)}e{randrange(-15,15)}")
+        return float(f"{choice('-+')}0.{randrange(1 << 20)}e{randrange(-15, 15)}")
 
     def schema_list(self, cls):
         while True:
-            scls = choice(self.types)
-            if scls is cls:
-                continue
-            if scls is float:
-                # TODO: float lists are currently adapted as decimal.
-                # There may be rounding errors or problems with inf.
+            # TODO: float lists are currently adapted as decimal.
+            # There may be rounding errors or problems with inf.
+            if (scls := choice(self.types)) is cls or scls is float:
                 continue
 
             # CRDB doesn't support arrays of json
             # https://github.com/cockroachdb/cockroach/issues/23468
             if self.conn.info.vendor == "CockroachDB" and scls in (Json, Jsonb):
                 continue
-
-            schema = self.make_schema(scls)
-            if schema is not None:
+            if (schema := self.make_schema(scls)) is not None:
                 break
 
         return (cls, schema)
@@ -583,8 +570,7 @@ class Faker:
 
         out: list[Range[Any]] = []
         for i in range(length):
-            r = self.make_Range((Range, spec[1]), **kwargs)
-            if r.isempty:
+            if (r := self.make_Range((Range, spec[1]), **kwargs)).isempty:
                 continue
             for r2 in out:
                 if overlap(r, r2):
@@ -839,8 +825,7 @@ class Faker:
         rec_types = [list, dict]
         scal_types = [type(None), int, JsonFloat, bool, str]
         if random() < container_chance:
-            cls = choice(rec_types)
-            if cls is list:
+            if (cls := choice(rec_types)) is list:
                 return [
                     self._make_json(container_chance=container_chance / 2.0)
                     for i in range(randrange(self.json_max_length))
@@ -932,7 +917,7 @@ class JsonFloat:
 
 
 def deep_import(name):
-    parts = Deque(name.split("."))
+    parts = deque(name.split("."))
     seen = []
     if not parts:
         raise ValueError("name must be a dot-separated name")
