@@ -3,7 +3,8 @@ from __future__ import annotations
 import logging
 import weakref
 from time import time
-from typing import Any, Dict
+from typing import Any
+from collections import Counter
 
 import pytest
 
@@ -11,7 +12,7 @@ import psycopg
 from psycopg.pq import TransactionStatus
 from psycopg.rows import Row, TupleRow, class_row
 
-from ..utils import Counter, assert_type, set_autocommit
+from ..utils import assert_type, set_autocommit, skip_free_threaded
 from ..acompat import AEvent, asleep, gather, skip_sync, spawn
 from .test_pool_common_async import delay_connection
 
@@ -24,6 +25,8 @@ except ImportError:
 if True:  # ASYNC
     pytestmark = [pytest.mark.anyio]
 
+PSYCOPG_VERSION = tuple(map(int, psycopg.__version__.split(".", 2)[:2]))
+
 
 async def test_default_sizes(dsn):
     async with pool.AsyncConnectionPool(dsn) as p:
@@ -43,7 +46,7 @@ async def test_bad_size(dsn, min_size, max_size):
         pool.AsyncConnectionPool(min_size=min_size, max_size=max_size)
 
 
-class MyRow(Dict[str, Any]):
+class MyRow(dict[str, Any]):
     pass
 
 
@@ -581,14 +584,14 @@ async def test_reconnect_after_grow_failed(proxy):
     async with pool.AsyncConnectionPool(
         proxy.client_dsn, min_size=4, reconnect_timeout=1.0, reconnect_failed=failed
     ) as p:
-        await ev.wait_timeout(2.0)
+        assert await ev.wait_timeout(2.0)
 
         with pytest.raises(pool.PoolTimeout):
             async with p.connection(timeout=0.5) as conn:
                 pass
 
         ev.clear()
-        await ev.wait_timeout(2.0)
+        assert await ev.wait_timeout(2.0)
 
         proxy.start()
 
@@ -618,7 +621,7 @@ async def test_refill_on_check(proxy):
 
         # Checking the pool will empty it
         await p.check()
-        await ev.wait_timeout(2.0)
+        assert await ev.wait_timeout(2.0)
         assert len(p._pool) == 0
 
         # Allow to connect again
@@ -859,6 +862,7 @@ async def test_check_max_lifetime(dsn):
 
 
 @pytest.mark.slow
+@skip_free_threaded("timing not accurate under the free-threaded build")
 async def test_stats_connect(proxy, monkeypatch):
     proxy.start()
     delay_connection(monkeypatch, 0.2)
@@ -968,7 +972,7 @@ async def test_cancellation_in_queue(dsn):
         tasks = [spawn(worker, (i,)) for i in range(nconns * 3)]
 
         # wait until the pool has served all the connections and clients are queued.
-        await ev.wait_timeout(3.0)
+        assert await ev.wait_timeout(3.0)
         for i in range(10):
             if p.get_stats().get("requests_queued", 0):
                 break
@@ -1020,30 +1024,56 @@ async def test_check_backoff(dsn, caplog, monkeypatch):
         want *= 2
 
 
+@pytest.mark.slow
+@pytest.mark.parametrize("status", ["ERROR", "INTRANS"])
+async def test_check_returns_an_ok_connection(dsn, status):
+    async def check(conn):
+        if status == "ERROR":
+            await conn.execute("wat")
+        elif status == "INTRANS":
+            await conn.execute("select 1")
+            1 / 0
+        else:
+            assert False
+
+    async with pool.AsyncConnectionPool(dsn, min_size=1, check=check) as p:
+        await p.wait(1.0)
+        with pytest.raises(pool.PoolTimeout):
+            conn = await p.getconn(0.5)
+
+        conn = list(p._pool)[0]
+        assert conn.info.transaction_status == TransactionStatus.IDLE
+
+
+class ReturningConnection(psycopg.AsyncConnection[Row]):
+    """
+    Test connection returning to the pool on close.
+
+    Verify that it's possible to override `close()` to act as `putconn()`.
+    which allows to use the psycopg pool in a sqlalchemy NullPool.
+
+    We cannot guarantee 100% that we will never break this implementation,
+    but we can keep awareness that we use it this way, maintain it on a
+    best-effort basis, and notify upstream if we are forced to break it.
+
+    https://github.com/sqlalchemy/sqlalchemy/discussions/12522
+    https://github.com/psycopg/psycopg/issues/1046
+    """
+
+    async def close(self) -> None:
+        if pool := getattr(self, "_pool", None):
+            # Connection currently checked out from the pool.
+            # Instead of closing it, return it to the pool.
+            await pool.putconn(self)
+        else:
+            # Connection not part of any pool, or currently into the pool.
+            # Close the connection for real.
+            await super().close()
+
+
 async def test_override_close(dsn):
-    # Verify that it's possible to override `close()` to act as `putconn()`.
-    # which allows to use the psycopg pool in a sqlalchemy NullPool.
-    #
-    # We cannot guarantee 100% that we will never break this implementation,
-    # but we can keep awareness that we use it this way, maintain it on a
-    # best-effort basis, and notify upstream if we are forced to break it.
-    #
-    # https://github.com/sqlalchemy/sqlalchemy/discussions/12522
-    # https://github.com/psycopg/psycopg/issues/1046
-
-    class MyConnection(psycopg.AsyncConnection[Row]):
-        async def close(self) -> None:
-            if pool := getattr(self, "_pool", None):
-                # Connection currently checked out from the pool.
-                # Instead of closing it, return it to the pool.
-                await pool.putconn(self)
-            else:
-                # Connection not part of any pool, or currently into the pool.
-                # Close the connection for real.
-                await super().close()
-
     async with pool.AsyncConnectionPool(
-        dsn, connection_class=MyConnection, min_size=2
+        dsn, connection_class=ReturningConnection, min_size=2
     ) as p:
         await p.wait()
         assert len(p._pool) == 2
@@ -1055,3 +1085,182 @@ async def test_override_close(dsn):
         assert len(p._pool) == 2
 
     assert conn.closed
+
+
+async def test_close_returns(dsn):
+
+    async with pool.AsyncConnectionPool(dsn, min_size=2, close_returns=True) as p:
+        await p.wait()
+        assert len(p._pool) == 2
+        conn = await p.getconn()
+        assert not conn.closed
+        assert len(p._pool) == 1
+        await conn.close()
+        assert not conn.closed
+        assert len(p._pool) == 2
+
+    assert conn.closed
+
+
+@pytest.mark.skipif(PSYCOPG_VERSION < (3, 3), reason="psycopg >= 3.3 behaviour")
+async def test_close_returns_custom_class(dsn):
+    class MyConnection(psycopg.AsyncConnection):
+        pass
+
+    async with pool.AsyncConnectionPool(
+        dsn, min_size=2, connection_class=MyConnection, close_returns=True
+    ) as p:
+        await p.wait()
+        conn = await p.getconn()
+        assert not conn.closed
+        assert len(p._pool) == 1
+        await conn.close()
+        assert not conn.closed
+        assert len(p._pool) == 2
+
+    assert conn.closed
+
+
+@pytest.mark.skipif(PSYCOPG_VERSION >= (3, 3), reason="psycopg < 3.3 behaviour")
+async def test_close_returns_custom_class_old(dsn):
+    class MyConnection(psycopg.AsyncConnection):
+        pass
+
+    with pytest.raises(TypeError, match="close_returns=True"):
+        pool.AsyncConnectionPool(dsn, connection_class=MyConnection, close_returns=True)
+
+
+@pytest.mark.slow
+@pytest.mark.timing
+@pytest.mark.skipif(PSYCOPG_VERSION < (3, 3), reason="psycopg >= 3.3 behaviour")
+async def test_close_returns_no_loop(dsn):
+    async with pool.AsyncConnectionPool(
+        dsn, min_size=1, close_returns=True, max_lifetime=0.05
+    ) as p:
+        conn = await p.getconn()
+        await asleep(0.1)
+        assert len(p._pool) == 0
+        await asleep(0.1)  # wait for the connection to expire
+        await conn.close()
+        await asleep(0.1)
+        assert len(p._pool) == 1
+        conn = await p.getconn()
+        await asleep(0.1)
+        assert len(p._pool) == 0
+        await conn.close()
+        await asleep(0.1)
+        assert len(p._pool) == 1
+
+
+@pytest.mark.slow
+@pytest.mark.timing
+async def test_override_close_no_loop_subclass(dsn):
+    async with pool.AsyncConnectionPool(
+        dsn, min_size=1, max_lifetime=0.05, connection_class=ReturningConnection
+    ) as p:
+        conn = await p.getconn()
+        await asleep(0.1)
+        assert len(p._pool) == 0
+        await asleep(0.1)  # wait for the connection to expire
+        await conn.close()
+        await asleep(0.1)
+        assert len(p._pool) == 1
+        conn = await p.getconn()
+        await asleep(0.1)
+        assert len(p._pool) == 0
+        await conn.close()
+        await asleep(0.1)
+        assert len(p._pool) == 1
+
+
+async def test_get_config_rotates_connections(dsn):
+
+    config_rotation_counter = 0
+
+    async def rotating_config():
+        nonlocal config_rotation_counter
+        config_rotation_counter += 1
+        return dsn
+
+    app_names = ["app-1", "app-2"]
+    kwargs_counter = 0
+
+    async def rotating_kwargs():
+        # Return a different application_name for each new connection
+        nonlocal kwargs_counter
+        kwargs_counter += 1
+        return {"application_name": app_names[kwargs_counter % len(app_names)]}
+
+    p = pool.AsyncConnectionPool(
+        conninfo=rotating_config,
+        kwargs=rotating_kwargs,
+        min_size=2,
+        max_lifetime=0.2,
+        open=False,
+    )
+
+    try:
+        await p.open()
+        await p.wait()
+
+        # Make sure we created two connections (rotating_config called twice)
+        assert config_rotation_counter == 2
+
+        # Acquire both connections and check application_name
+        async with p.connection() as conn1, p.connection() as conn2:
+            row1 = await conn1.execute("SHOW application_name")
+            row2 = await conn2.execute("SHOW application_name")
+
+            name1 = await row1.fetchone()
+            assert (
+                name1 is not None
+            ), "first call to SHOW application_name returned no rows"
+            assert name1[0] in app_names
+
+            name2 = await row2.fetchone()
+            assert (
+                name2 is not None
+            ), "second call to SHOW application_name returned no rows"
+            assert name2[0] in app_names
+
+            # Make sure that names are different.
+            assert name1 != name2
+
+    finally:
+        await p.close()
+
+
+@pytest.mark.slow
+async def test_get_config_raise_exception(dsn, caplog):
+
+    async def failing_conninfo():
+        raise RuntimeError("cannot build conninfo")
+
+    async def failing_kwargs():
+        raise RuntimeError("cannot build kwargs")
+
+    p = pool.AsyncConnectionPool(
+        conninfo=failing_conninfo,
+        kwargs=failing_kwargs,
+        min_size=1,
+        max_lifetime=0.1,
+        open=False,
+        reconnect_timeout=1.0,
+    )
+
+    with caplog.at_level("WARNING"):
+        try:
+            await p.open()
+            with pytest.raises(pool.PoolTimeout):
+                await p.wait(timeout=2.0)
+        finally:
+            await p.close()
+
+    # Ensure log contains "reconnection attempt" warning`
+    reconnection_warnings = [
+        rec for rec in caplog.records if "reconnection attempt" in rec.message.lower()
+    ]
+
+    assert reconnection_warnings, "Expected reconnection attempt logs"
+    # Make sure that we saw not too many (backoff works)
+    assert len(reconnection_warnings) < 5, "Too many attempts (likely busyloop)"
