@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 import logging
-from typing import Any, Dict
+from typing import Any
 
 import pytest
 from packaging.version import parse as ver  # noqa: F401  # used in skipif
@@ -41,7 +41,7 @@ async def test_bad_size(dsn, min_size, max_size):
         pool.AsyncNullConnectionPool(min_size=min_size, max_size=max_size)
 
 
-class MyRow(Dict[str, Any]):
+class MyRow(dict[str, Any]):
     pass
 
 
@@ -174,6 +174,7 @@ async def test_reset(dsn):
             cur = await conn.execute("show timezone")
             assert (await cur.fetchone()) == ("UTC",)
             pids.append(conn.info.backend_pid)
+            assert conn._pool is p
 
     async with pool.AsyncNullConnectionPool(dsn, max_size=1, reset=reset) as p:
         async with p.connection() as conn:
@@ -185,6 +186,7 @@ async def test_reset(dsn):
             assert resets == 0
             await conn.execute("set timezone to '+2:00'")
             pids.append(conn.info.backend_pid)
+            assert conn._pool is p
 
         await gather(t)
         await p.wait()
@@ -460,8 +462,6 @@ async def test_cancellation_in_queue(dsn):
         async def worker(i):
             try:
                 logging.info("worker %s started", i)
-                nonlocal got_conns
-
                 async with p.connection() as conn:
                     logging.info("worker %s got conn", i)
                     cur = await conn.execute("select 1")
@@ -481,7 +481,7 @@ async def test_cancellation_in_queue(dsn):
         tasks = [spawn(worker, (i,)) for i in range(nconns * 3)]
 
         # wait until the pool has served all the connections and clients are queued.
-        await ev.wait_timeout(3.0)
+        assert await ev.wait_timeout(3.0)
         for i in range(10):
             if p.get_stats().get("requests_queued", 0):
                 break
@@ -499,3 +499,12 @@ async def test_cancellation_in_queue(dsn):
         async with p.connection() as conn:
             cur = await conn.execute("select 1")
             assert await cur.fetchone() == (1,)
+
+
+async def test_close_returns(dsn):
+    # Mostly test the interface; close is close even if it goes via putconn().
+    async with pool.AsyncNullConnectionPool(dsn, close_returns=True) as p:
+        conn = await p.getconn()
+        assert not conn.closed
+        await conn.close()
+        assert conn.closed
