@@ -8,8 +8,9 @@ import logging
 import weakref
 import contextlib
 from select import select
-from typing import TYPE_CHECKING, Iterator
+from typing import TYPE_CHECKING
 from functools import partial
+from collections.abc import Iterator
 
 import pytest
 
@@ -18,8 +19,6 @@ import psycopg.generators
 from psycopg import pq
 from psycopg.conninfo import make_conninfo
 
-from ..fix_crdb import crdb_anydb
-
 if TYPE_CHECKING:
     from psycopg.pq.abc import PGcancelConn, PGconn
 
@@ -52,7 +51,7 @@ def test_connectdb(dsn):
     assert conn.status == pq.ConnStatus.OK, conn.error_message
 
 
-@crdb_anydb
+@pytest.mark.crdb("skip", reason="can connect to any db name")
 def test_connectdb_error(dsn):
     conn = pq.PGconn.connect(make_conninfo(dsn, dbname="nosuchdb").encode())
     assert conn.status == pq.ConnStatus.BAD
@@ -305,6 +304,20 @@ def test_protocol_version(pgconn):
         pgconn.protocol_version
 
 
+@pytest.mark.libpq(">= 18")
+def test_full_protocol_version(pgconn):
+    assert pgconn.full_protocol_version >= 30000
+    pgconn.finish()
+    with pytest.raises(psycopg.OperationalError):
+        pgconn.full_protocol_version
+
+
+@pytest.mark.libpq("< 18")
+def test_full_protocol_version_notimpl(pgconn):
+    with pytest.raises(psycopg.NotSupportedError):
+        pgconn.full_protocol_version
+
+
 def test_server_version(pgconn):
     assert pgconn.server_version >= 90400
     pgconn.finish()
