@@ -13,16 +13,17 @@ from __future__ import annotations
 import asyncio
 import logging
 import argparse
-from typing import Any, Iterator, Sequence
+from typing import Any
 from functools import partial
 from contextlib import contextmanager
+from collections import deque
+from collections.abc import Iterator, Sequence
 
 from psycopg import AsyncConnection, Connection
 from psycopg import errors as e
 from psycopg import pq, waiting
 from psycopg.pq import DiagnosticField, Format
 from psycopg.abc import PipelineCommand
-from psycopg._compat import Deque
 from psycopg.generators import pipeline_communicate
 
 psycopg_logger = logging.getLogger("psycopg")
@@ -93,17 +94,13 @@ class LoggingPGconn:
         self._logger.info("sent prepared '%s' with %s", name.decode(), param_values)
 
     def send_prepare(
-        self,
-        name: bytes,
-        command: bytes,
-        param_types: Sequence[int] | None = None,
+        self, name: bytes, command: bytes, param_types: Sequence[int] | None = None
     ) -> None:
         self._pgconn.send_prepare(name, command, param_types)
         self._logger.info("prepare %s as '%s'", command.decode(), name.decode())
 
     def get_result(self) -> pq.abc.PGresult | None:
-        r = self._pgconn.get_result()
-        if r is not None:
+        if (r := self._pgconn.get_result()) is not None:
             self._logger.info("got %s result", pq.ExecStatus(r.status).name)
         return r
 
@@ -111,7 +108,7 @@ class LoggingPGconn:
 @contextmanager
 def prepare_pipeline_demo_pq(
     pgconn: LoggingPGconn, rows_to_send: int, logger: logging.Logger
-) -> Iterator[tuple[Deque[PipelineCommand], Deque[str]]]:
+) -> Iterator[tuple[deque[PipelineCommand], deque[str]]]:
     """Set up pipeline demo with initial queries and yield commands and
     results queue for pipeline_communicate().
     """
@@ -137,8 +134,8 @@ def prepare_pipeline_demo_pq(
         ),
     ]
 
-    commands = Deque[PipelineCommand]()
-    results_queue = Deque[str]()
+    commands = deque[PipelineCommand]()
+    results_queue = deque[str]()
 
     for qname, query in setup_queries:
         if qname == "prepare":
@@ -190,8 +187,7 @@ def pipeline_demo_pq(rows_to_send: int, logger: logging.Logger) -> None:
     ):
         while results_queue:
             fetched = waiting.wait(
-                pipeline_communicate(pgconn, commands),
-                pgconn.socket,
+                pipeline_communicate(pgconn, commands), pgconn.socket
             )
             assert not commands, commands
             for results in fetched:
@@ -213,8 +209,7 @@ async def pipeline_demo_pq_async(rows_to_send: int, logger: logging.Logger) -> N
     ):
         while results_queue:
             fetched = await waiting.wait_async(
-                pipeline_communicate(pgconn, commands),
-                pgconn.socket,
+                pipeline_communicate(pgconn, commands), pgconn.socket
             )
             assert not commands, commands
             for results in fetched:
