@@ -13,14 +13,17 @@ from psycopg import errors as e
 from psycopg import pq
 from psycopg.rows import tuple_row
 from psycopg.conninfo import conninfo_to_dict, timeout_from_conninfo
+from psycopg._conninfo_utils import get_param
 
-from .acompat import asleep, is_async, skip_async, skip_sync
+from .acompat import asleep, skip_async, skip_sync
 from .test_adapt import make_bin_dumper, make_dumper
 from ._test_cursor import my_row_factory
 from ._test_connection import testctx  # noqa: F401  # fixture
 from ._test_connection import conninfo_params_timeout, tx_params, tx_params_isolation
 from ._test_connection import tx_values_map
 
+MULTI_FAILURE_MESSAGE = "Multiple connection attempts failed. All failures were:"
+
 
 async def test_connect(aconn_cls, dsn):
     conn = await aconn_cls.connect(dsn)
@@ -29,9 +32,48 @@ async def test_connect(aconn_cls, dsn):
     await conn.close()
 
 
-async def test_connect_bad(aconn_cls):
+@pytest.mark.crdb("skip", reason="can connect to any db name")
+async def test_connect_bad(aconn_cls, dsn):
     with pytest.raises(psycopg.OperationalError):
-        await aconn_cls.connect("dbname=nosuchdb")
+        await aconn_cls.connect(dsn, dbname="nosuchdb")
+
+
+@pytest.mark.slow
+async def test_connect_error_single_host_original_message_preserved(aconn_cls, proxy):
+    with proxy.deaf_listen():
+        with pytest.raises(psycopg.OperationalError) as e:
+            await aconn_cls.connect(proxy.client_dsn, connect_timeout=2)
+
+    msg = str(e)
+    assert "connection timeout expired" in msg
+    assert MULTI_FAILURE_MESSAGE not in msg
+
+
+@pytest.mark.slow
+async def test_connect_error_multi_hosts_each_message_preserved(aconn_cls):
+    args = {
+        # IPv4 address blocks reserved for documentation.
+        # https://datatracker.ietf.org/doc/rfc5737/
+        "host": "192.0.2.1,198.51.100.1",
+        "port": "1234,5678",
+    }
+    with pytest.raises(psycopg.OperationalError) as e:
+        await aconn_cls.connect(**args, connect_timeout=2)
+
+    msg = str(e.value)
+    assert MULTI_FAILURE_MESSAGE in msg
+
+    host1, host2 = args["host"].split(",")
+    port1, port2 = args["port"].split(",")
+
+    msg_lines = msg.splitlines()
+
+    expected_host1 = f"host: '{host1}', port: '{port1}', hostaddr: '{host1}'"
+    expected_host2 = f"host: '{host2}', port: '{port2}', hostaddr: '{host2}'"
+    expected_error = "connection timeout expired"
+
+    assert any(expected_host1 in line and expected_error in line for line in msg_lines)
+    assert any(expected_host2 in line and expected_error in line for line in msg_lines)
 
 
 async def test_connect_str_subclass(aconn_cls, dsn):
@@ -135,42 +177,39 @@ async def test_cursor_closed(aconn):
 # TODO: the INERROR started failing in the C implementation in Python 3.12a7
 # compiled with Cython-3.0.0b3, not before.
 @pytest.mark.slow
-@pytest.mark.xfail(
-    pq.__impl__ in ("c", "binary")
-    and sys.version_info[:2] == (3, 12)
-    and not is_async(__name__),
-    reason="Something with Exceptions, C, Python 3.12",
-)
 async def test_connection_warn_close(aconn_cls, dsn, recwarn, gc_collect):
-    conn = await aconn_cls.connect(dsn)
-    await conn.close()
-    del conn
+    # First create all the connections to test, to avoid some to reuse the same
+    # address, resulting in an omitted warning.
+    conn1 = await aconn_cls.connect(dsn)
+    conn2 = await aconn_cls.connect(dsn)
+    conn3 = await aconn_cls.connect(dsn)
+    conn4 = await aconn_cls.connect(dsn)
+
+    async with await aconn_cls.connect(dsn) as conn5:
+        pass
+    del conn5
     assert not recwarn, [str(w.message) for w in recwarn.list]
 
-    conn = await aconn_cls.connect(dsn)
-    del conn
-    gc_collect()
-    assert "IDLE" in str(recwarn.pop(ResourceWarning).message)
+    await conn1.close()
+    del conn1
+    assert not recwarn, [str(w.message) for w in recwarn.list]
 
-    conn = await aconn_cls.connect(dsn)
-    await conn.execute("select 1")
-    del conn
+    del conn2
     gc_collect()
-    assert "INTRANS" in str(recwarn.pop(ResourceWarning).message)
+    assert aconn_cls.__name__ in str(recwarn.pop(ResourceWarning).message)
+
+    await conn3.execute("select 1")
+    del conn3
+    gc_collect()
+    assert aconn_cls.__name__ in str(recwarn.pop(ResourceWarning).message)
 
-    conn = await aconn_cls.connect(dsn)
     try:
-        await conn.execute("select wat")
+        await conn4.execute("select wat")
     except psycopg.ProgrammingError:
         pass
-    del conn
+    del conn4
     gc_collect()
-    assert "INERROR" in str(recwarn.pop(ResourceWarning).message)
-
-    async with await aconn_cls.connect(dsn) as conn:
-        pass
-    del conn
-    assert not recwarn, [str(w.message) for w in recwarn.list]
+    assert aconn_cls.__name__ in str(recwarn.pop(ResourceWarning).message)
 
 
 @pytest.mark.usefixtures("testctx")
@@ -463,6 +502,14 @@ async def test_connect_args(
     await conn.close()
 
 
+async def test_resolve_error_host(aconn_cls):
+    host = "nosuchhost.example.com"
+    with pytest.raises(psycopg.OperationalError) as ex:
+        await aconn_cls.connect(host=host)
+
+    assert host in str(ex.value)
+
+
 @pytest.mark.parametrize(
     "args, kwargs, exctype",
     [
@@ -902,7 +949,7 @@ async def test_right_exception_on_session_timeout(aconn):
         want_ex = psycopg.OperationalError
 
     await aconn.execute("SET SESSION idle_in_transaction_session_timeout = 100")
-    await asleep(0.2)
+    await asleep(1)
     with pytest.raises(want_ex) as ex:
         await aconn.execute("SELECT * from pg_tables")
 
@@ -927,3 +974,16 @@ async def test_connect_tsa_bad(aconn_cls, dsn, mode):
     params = conninfo_to_dict(dsn, target_session_attrs=mode)
     with pytest.raises(psycopg.OperationalError, match=mode):
         await aconn_cls.connect(**params)
+
+
+@pytest.mark.libpq(">= 16")
+@pytest.mark.skipif(pq.__impl__ != "python", reason="can't monkeypatch C module")
+async def test_implicit_gssapi_warning(aconn_cls, dsn, recwarn, monkeypatch):
+    if get_param(conninfo_to_dict(dsn), "gssencmode"):
+        pytest.skip("gssencmode parameter explicitly set in test connection string")
+
+    monkeypatch.setattr(pq.PGconn, "used_gssapi", lambda: True)
+    async with await aconn_cls.connect(dsn):
+        pass
+
+    assert "gssencmode" in str(recwarn.pop(RuntimeWarning).message)
