@@ -9,15 +9,16 @@ import pytest
 import psycopg
 from psycopg import errors as e
 from psycopg import pq, sql
+from psycopg.abc import Buffer
 from psycopg.copy import AsyncCopy, AsyncLibpqWriter, AsyncQueuedLibpqWriter
-from psycopg.adapt import PyFormat
+from psycopg.adapt import Dumper, PyFormat
 from psycopg.types import TypeInfo
 from psycopg.types.hstore import register_hstore
 from psycopg.types.numeric import Int4
 
 from .utils import eur
-from .acompat import alist
-from ._test_copy import sample_binary  # noqa
+from .acompat import AEvent, alist, gather, spawn
+from ._test_copy import sample_binary  # noqa: F401
 from ._test_copy import AsyncFileWriter, ensure_table_async, py_to_raw
 from ._test_copy import sample_binary_rows, sample_records, sample_tabledef
 from ._test_copy import sample_text, sample_values, special_chars
@@ -120,6 +121,37 @@ async def test_rows(aconn, format):
     assert aconn.info.transaction_status == pq.TransactionStatus.INTRANS
 
 
+@pytest.mark.parametrize("format", pq.Format)
+async def test_set_types(aconn, format):
+    sample = ({"foo": "bar"}, 123)
+    cur = aconn.cursor()
+    await ensure_table_async(cur, "id serial primary key, data jsonb, data2 bigint")
+    async with cur.copy(
+        f"copy copy_in (data, data2) from stdin (format {format.name})"
+    ) as copy:
+        copy.set_types(["jsonb", "bigint"])
+        await copy.write_row(sample)
+    await cur.execute("select data, data2 from copy_in")
+    data = await cur.fetchone()
+    assert data == sample
+
+
+@pytest.mark.parametrize("format", pq.Format)
+@pytest.mark.parametrize("use_set_types", [True, False])
+async def test_rowlen_mismatch(aconn, format, use_set_types):
+    samples = [["foo", "bar"], ["foo", "bar", "baz"]]
+    cur = aconn.cursor()
+    await ensure_table_async(cur, "id serial primary key, data text, data2 text")
+    with pytest.raises(psycopg.DataError):
+        async with cur.copy(
+            f"copy copy_in (data, data2) from stdin (format {format.name})"
+        ) as copy:
+            if use_set_types:
+                copy.set_types(["text", "text"])
+            for row in samples:
+                await copy.write_row(row)
+
+
 async def test_set_custom_type(aconn, hstore):
     command = """copy (select '"a"=>"1", "b"=>"2"'::hstore) to stdout"""
     cur = aconn.cursor()
@@ -148,10 +180,7 @@ async def test_copy_out_allchars(aconn, format):
     )
     async with cur.copy(query) as copy:
         copy.set_types(["text"])
-        while True:
-            row = await copy.read_row()
-            if not row:
-                break
+        while row := (await copy.read_row()):
             assert len(row) == 1
             rows.append(row[0])
 
@@ -165,10 +194,7 @@ async def test_read_row_notypes(aconn, format):
         f"copy ({sample_values}) to stdout (format {format.name})"
     ) as copy:
         rows = []
-        while True:
-            row = await copy.read_row()
-            if not row:
-                break
+        while row := (await copy.read_row()):
             rows.append(row)
 
     ref = [tuple(py_to_raw(i, format) for i in record) for record in sample_records]
@@ -476,6 +502,74 @@ async def test_copy_in_records_binary(aconn, format):
     assert data == [(1, None, "hello"), (2, None, "world")]
 
 
+class StrictIntDumper(Dumper):
+    oid = psycopg.adapters.types["int4"].oid
+
+    def dump(self, obj: int) -> Buffer:
+        if type(obj) is not int:
+            raise TypeError(f"bad type: {obj!r}")
+        return str(obj).encode()
+
+
+async def test_copy_in_text_no_pinning(aconn):
+    cur = aconn.cursor()
+    cur.adapters.register_dumper(int, StrictIntDumper)
+
+    cols = [
+        "col1 serial primary key",
+        "col2 int",
+        "col3 int",
+        "col4 double precision",
+        "col5 double precision",
+    ]
+    await ensure_table_async(cur, ",".join(cols))
+
+    async with cur.copy(
+        "copy copy_in (col2,col3,col4,col5) from stdin (format text)"
+    ) as copy:
+        # no pinned dumpers: type check & cast done on postgres side
+        # allows to mix castable reprs more freely
+        # slower than pinned, late errors from postgres jeopardizing copy cursor
+        await copy.write_row([1, "2", 3, "4.1"])
+        await copy.write_row(["1", 2, 3.0, 4])
+
+    await cur.execute("select col2,col3,col4,col5 from copy_in order by 1")
+    data = await cur.fetchall()
+    assert data == [(1, 2, 3, 4.1), (1, 2, 3, 4)]
+
+
+async def test_copy_in_text_pinned(aconn):
+    cur = aconn.cursor()
+    cur.adapters.register_dumper(int, StrictIntDumper)
+
+    cols = [
+        "col1 serial primary key",
+        "col2 int",
+        "col3 int",
+        "col4 double precision",
+        "col5 double precision",
+    ]
+    await ensure_table_async(cur, ",".join(cols))
+
+    async with cur.copy(
+        "copy copy_in (col2,col3,col4,col5) from stdin (format text)"
+    ) as copy:
+        # pinned dumpers from set_types: type check & cast done on psycopg side
+        # much faster, allows catching errors early without postgres involvement
+        copy.set_types(["int4", "int4", "double precision", "double precision"])
+        await copy.write_row([1, 2, 3, 4.1])
+        with pytest.raises(
+            (e.DataError, TypeError)
+        ):  # FIXME: should errors from dumpers be harmonized?
+            await copy.write_row([1.0, 2, 3, 4.1])
+        with pytest.raises((e.DataError, TypeError)):
+            await copy.write_row([1, "2", 3, 4.1])
+
+    await cur.execute("select col2,col3,col4,col5 from copy_in order by 1")
+    data = await cur.fetchall()
+    assert data == [(1, 2, 3, 4.1)]
+
+
 async def test_copy_in_allchars(aconn):
     cur = aconn.cursor()
     await ensure_table_async(cur, sample_tabledef)
@@ -663,6 +757,37 @@ async def test_description(aconn):
         assert cur.description[2].name == "column_3"
 
 
+async def test_binary_partial_row(aconn):
+    cur = aconn.cursor()
+    await ensure_table_async(cur, "id serial primary key, num int4, arr int4[][]")
+    with pytest.raises(
+        psycopg.DataError, match="nested lists have inconsistent depths"
+    ):
+        async with cur.copy(
+            "copy copy_in (num, arr) from stdin (format binary)"
+        ) as copy:
+            copy.set_types(["int4", "int4[]"])
+            await copy.write_row([15, None])
+            await copy.write_row([16, [[None], None]])
+
+
+@pytest.mark.parametrize("format", pq.Format)
+async def test_clean_buffer_on_error(aconn, format):
+    cur = aconn.cursor()
+    await ensure_table_async(cur, "id serial primary key, num int4, obj jsonb")
+    async with cur.copy(
+        f"copy copy_in (num, obj) from stdin (format {format.name})"
+    ) as copy:
+        copy.set_types(["int4", "jsonb"])
+        await copy.write_row([15, {}])
+        with pytest.raises(TypeError):
+            await copy.write_row([16, 1j])
+        await copy.write_row([17, []])
+
+    await cur.execute("select num, obj from copy_in order by id")
+    assert (await cur.fetchall()) == [(15, {}), (17, [])]
+
+
 @pytest.mark.parametrize(
     "format, buffer",
     [(pq.Format.TEXT, "sample_text"), (pq.Format.BINARY, "sample_binary")],
@@ -751,17 +876,13 @@ async def test_copy_to_leaks(aconn_cls, dsn, faker, fmt, set_types, method, gc):
                         copy.set_types(faker.types_names)
 
                     if method == "read":
-                        while True:
-                            tmp = await copy.read()
-                            if not tmp:
-                                break
+                        while await copy.read():
+                            pass
                     elif method == "iter":
                         await alist(copy)
                     elif method == "row":
-                        while True:
-                            tmp = await copy.read_row()
-                            if tmp is None:
-                                break
+                        while (await copy.read_row()) is not None:
+                            pass
                     elif method == "rows":
                         await alist(copy.rows())
 
@@ -851,6 +972,105 @@ async def test_copy_table_across(aconn_cls, dsn, faker, mode):
             faker.assert_record(got, want)
 
 
+async def test_copy_concurrency(aconn):
+    """
+    Test that copy operations hold the connection lock for the entire operation.
+
+    This test verifies the fix for the concurrency issue where AsyncCursor.copy()
+    was not holding the connection lock throughout the copy context, allowing
+    concurrent operations to interfere.
+    """
+    await aconn.execute("create temp table copy_concurrency_test (id int, data text)")
+
+    # Events to coordinate execution between copy task and workers
+    copy_entered = AEvent()
+    wrote_first = AEvent()
+    wrote_second = AEvent()
+    can_proceed = AEvent()
+
+    # Track execution order to verify workers run after copy completes
+    execution_log = []
+
+    async def copy_task():
+        """Copy task that writes two rows with controlled pauses."""
+        cur = aconn.cursor()
+        async with cur.copy("copy copy_concurrency_test from stdin") as copy:
+            # Pause after entering copy context
+            execution_log.append("entered_copy")
+            copy_entered.set()
+            await can_proceed.wait()
+
+            # Write first row and pause
+            await copy.write_row((1, "first"))
+            execution_log.append("wrote_row_1")
+            wrote_first.set()
+            await can_proceed.wait()
+
+            # Write second row and pause
+            await copy.write_row((2, "second"))
+            execution_log.append("wrote_row_2")
+            wrote_second.set()
+            await can_proceed.wait()
+
+        # Copy context exited, lock should now be released
+        execution_log.append("exited_copy")
+
+    async def worker_task():
+        """
+        Worker that attempts to execute a query on a different cursor.
+        Should block until copy completes due to connection lock.
+        """
+        # Try to execute on another cursor - this should block until copy exits
+        worker_cur = aconn.cursor()
+        await worker_cur.execute("select 1")
+        execution_log.append("worker_completed")
+
+    # Start the copy task
+    t_copy = spawn(copy_task)
+
+    # Wait for copy to enter, then spawn first worker
+    await copy_entered.wait()
+    t_worker1 = spawn(worker_task)
+
+    # Allow copy to proceed to write first row
+    can_proceed.set()
+    can_proceed.clear()
+    await wrote_first.wait()
+
+    # Spawn second worker after first row
+    t_worker2 = spawn(worker_task)
+
+    # Allow copy to proceed to write second row
+    can_proceed.set()
+    can_proceed.clear()
+    await wrote_second.wait()
+
+    # Spawn third worker after second row
+    t_worker3 = spawn(worker_task)
+
+    # Allow copy to exit
+    can_proceed.set()
+
+    # Wait for all tasks to complete
+    await gather(t_copy, t_worker1, t_worker2, t_worker3)
+
+    # Verify the data was written correctly
+    cur = await aconn.execute("select * from copy_concurrency_test order by id")
+    rows = await cur.fetchall()
+    assert rows == [(1, "first"), (2, "second")]
+
+    # Verify that all workers completed AFTER copy exited
+    assert execution_log == [
+        "entered_copy",
+        "wrote_row_1",
+        "wrote_row_2",
+        "exited_copy",
+        "worker_completed",
+        "worker_completed",
+        "worker_completed",
+    ]
+
+
 class DataGenerator:
     def __init__(self, conn, nrecs, srec, offset=0, block_size=8192):
         self.conn = conn
@@ -878,10 +1098,7 @@ class DataGenerator:
 
     def blocks(self):
         f = self.file()
-        while True:
-            block = f.read(self.block_size)
-            if not block:
-                break
+        while block := f.read(self.block_size):
             yield block
 
     async def assert_data(self):
@@ -894,10 +1111,7 @@ class DataGenerator:
 
     def sha(self, f):
         m = hashlib.sha256()
-        while True:
-            block = f.read()
-            if not block:
-                break
+        while block := f.read():
             if isinstance(block, str):
                 block = block.encode()
             m.update(block)
