@@ -86,15 +86,11 @@ async def test_leak(aconn_cls, dsn, faker, fmt, fmt_out, fetch, row_factory, gc)
                 await cur.execute(faker.select_stmt)
 
                 if fetch == "one":
-                    while True:
-                        tmp = await cur.fetchone()
-                        if tmp is None:
-                            break
+                    while (await cur.fetchone()) is not None:
+                        pass
                 elif fetch == "many":
-                    while True:
-                        tmp = await cur.fetchmany(3)
-                        if not tmp:
-                            break
+                    while await cur.fetchmany(3):
+                        pass
                 elif fetch == "all":
                     await cur.fetchall()
                 elif fetch == "iter":
