@@ -15,6 +15,7 @@ from psycopg.types.string import StrDumper
 
 from .utils import eur
 from .fix_crdb import crdb_encoding, crdb_scs_off
+from .test_adapt import make_dumper
 
 
 @pytest.mark.parametrize(
@@ -294,7 +295,9 @@ class TestIdentifier:
     def test_as_bytes(self, conn, args, want, enc):
         want = want.encode(enc)
         conn.execute(f"set client_encoding to {py2pgenc(enc).decode()}")
-        assert sql.Identifier(*args).as_bytes(conn) == want
+        got = sql.Identifier(*args).as_bytes(conn)
+        assert isinstance(got, bytes)
+        assert got == want
 
     @pytest.mark.parametrize("args, want, enc", _as_bytes_params)
     def test_as_bytes_no_conn(self, conn, args, want, enc):
@@ -345,6 +348,7 @@ class TestLiteral:
     @pytest.mark.parametrize("obj, want", _params)
     def test_as_bytes(self, conn, obj, want):
         got = sql.Literal(obj).as_bytes(conn)
+        assert isinstance(got, bytes)
         if isinstance(obj, str):
             got = no_e(got)
         assert got == want.encode()
@@ -497,8 +501,8 @@ class TestSQL:
         assert isinstance(obj, sql.Composed)
         assert obj.as_string(conn) == """'foo', 'bar', 42"""
 
-        obj = sql.SQL(", ").join([])
-        assert obj == sql.Composed([])
+        obj2 = sql.SQL(", ").join([])
+        assert obj2 == sql.Composed([])
 
     def test_as_string(self, conn):
         assert sql.SQL("foo").as_string(conn) == "foo"
@@ -509,7 +513,9 @@ class TestSQL:
         if encoding:
             conn.execute(f"set client_encoding to {encoding}")
 
-        assert sql.SQL(eur).as_bytes(conn) == eur.encode(encoding)
+        got = sql.SQL(eur).as_bytes(conn)
+        assert isinstance(got, bytes)
+        assert got == eur.encode(encoding)
 
     def test_no_conn(self):
         assert sql.SQL(eur).as_string() == eur
@@ -629,6 +635,7 @@ class TestPlaceholder:
     @pytest.mark.parametrize("format", PyFormat)
     def test_as_bytes(self, conn, format):
         ph = sql.Placeholder(format=format)
+        assert isinstance(ph.as_bytes(), bytes)
         assert ph.as_bytes(conn) == f"%{format.value}".encode()
         assert ph.as_bytes() == f"%{format.value}".encode()
 
@@ -637,6 +644,48 @@ class TestPlaceholder:
         assert ph.as_bytes() == f"%(foo){format.value}".encode()
 
 
+def test_as_string():
+    query = sql.as_string(sql.SQL("select {}").format("foo"))
+    assert query == no_e("select 'foo'")
+
+
+def test_as_string_context(conn):
+    conn.adapters.register_dumper(str, make_dumper("1"))
+    query = sql.as_string(sql.SQL("select {}").format("foo"), context=conn)
+    assert query == no_e("select 'foo1'")
+
+
+def test_as_string_literal():
+    got = sql.as_string("query")
+    assert got == no_e("'query'")
+
+    got = sql.as_string(dt.date(1970, 1, 1))
+    assert got == no_e("'1970-01-01'::date")
+
+
+def test_as_bytes():
+    query = sql.as_bytes(sql.SQL("select {}").format("foo"))
+    assert isinstance(query, bytes)
+    assert query == no_e(b"select 'foo'")
+
+
+def test_as_bytes_context(conn):
+    conn.adapters.register_dumper(str, make_dumper("1"))
+    query = sql.as_bytes(sql.SQL("select {}").format("foo"), context=conn)
+    assert isinstance(query, bytes)
+    assert query == no_e(b"select 'foo1'")
+
+
+def test_as_bytes_literal():
+    got = sql.as_bytes("query")
+    assert isinstance(got, bytes)
+    assert got == no_e(b"'query'")
+
+    got = sql.as_bytes(dt.date(1970, 1, 1))
+    assert isinstance(got, bytes)
+    assert got == no_e(b"'1970-01-01'::date")
+
+
 class TestValues:
     def test_null(self, conn):
         assert isinstance(sql.NULL, sql.SQL)
